/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
/*
 * mazen-ng
 * Copyright (C) Jacob Zimmermann 2009 <jacob@jzimm.net>
 * 
 * mazen-ng is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * mazen-ng is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#pragma implementation

#include "dungeon-maze.h"
#include <algorithm>

using namespace Glib;

DungeonMaze::DungeonMaze (int width, int height, int n_trees, int n_floating_trees):
	DungeonMazeType (width, height),
	entrance (0, (height - 1) / 2) {
		exit_dir = GridMaze::LEFT;
		
		cw = __dm_get_cc (width);
		ch = __dm_get_cc (height);
		
		cx = (width - 1 - cw) / 2;
		cy = (height - 1 - ch) / 2;
		exit.set (cx + cw, cy + ch / 2);

		// create border wall
		for (int x = 0; x < width - 1; x++) {
			set_horiz (x, 0);
			set_horiz (x, height - 1);
		}
		for (int y = 0; y < height - 1; y++) {
			set_vert (0, y);
			set_vert (width - 1, y);
		}
		Wall outer;
		outer.push_back (entrance);
		outer.push_back (xy (0, 0));
		outer.push_back (xy (width - 1, 0));
		outer.push_back (xy (width - 1, height - 1));
		outer.push_back (xy (0, height - 1));
		outer.push_back (entrance.up ());
		walls.push_back (outer);

		// create inner room
		for (int k = 0; k < cw; k++) {
			set_horiz (cx + k, cy);
			set_horiz (cx + k, cy + ch);
		}
		for (int k = 0; k < ch; k++) {
			set_vert (cx, cy + k);
			set_vert (cx + cw, cy + k);
		}
		Wall inner;
		inner.push_back (exit);
		inner.push_back (xy (cx + cw, cy));
		inner.push_back (xy (cx, cy));
		inner.push_back (xy (cx, cy + ch));
		inner.push_back (xy (cx + cw, cy + ch));
		inner.push_back (exit.up ());
		walls.push_back (inner);

		// entrance and exit
		clear_vert (entrance);
		clear_vert (exit);

		// No trees inside the central room
		for (int x = cx + 1; x < cx + cw; x++)
			for (int y = cy + 1; y < cy + ch; y++)
				set_no_no (x, y);
		
		// border trees coordinates
		std::vector<xy> t;
		for (int x = 0; x < width - 2; x++) {
			t.push_back (xy (x + 1, 0));
			t.push_back (xy (x + 1, height - 1));
		}
		for (int y = 0; y < height - 2; y++) {
			t.push_back (xy (0, y + 1));
			t.push_back (xy (width - 1, y + 1));
		}

		// inner room-based trees
		for (int x = cx; x < cx + cw; x++) {
			t.push_back (xy (x, cy));
			t.push_back (xy (x, cy + cw));
		}
		for (int y = cy + 1; y < cy + ch; y++) {
			t.push_back (xy (cx, y));
			t.push_back (xy (cx + ch, y));
		}
		
		// create trees		
		std::random_shuffle (t.begin (), t.end ());
		for (int k = 0; k < n_trees; k++)
			tree_positions.push_back (t[k]);

		// create floating trees
		int x, y;
		for (int k = 0; k < n_floating_trees; k++) {
			x = g_random_int_range (1, width - 1);
			y = g_random_int_range (1, height - 1);
			if (is_clear (x, y))
				tree_positions.push_back (xy (x, y));
		}

		generate();
	}

xy DungeonMaze::get_entrance () const { return entrance; }
xy DungeonMaze::get_exit () const { return exit; }
