/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
/*
 * mazen-ng
 * Copyright (C) Jacob Zimmermann 2009 <jacob@jzimm.net>
 * 
 * mazen-ng is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * mazen-ng is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#pragma implementation

#include "dungeon-type-view.h"
#include "dungeon-maze-type.h"
#include "util.h"

DungeonTypeView::DungeonTypeView (): GridView () {
}

ViewPtr DungeonTypeView::create () {
	return ViewPtr (new DungeonTypeView ());
}

void DungeonTypeView::set_size (int width, int height) {
	GridView::set_size (width / 2, height);

	this->width = width;
	this->height = height;
	maze_x = (double) width / 4;
	double real_maze_height = ((double) maze->height () - 1) * zfact;
	maze_y = (double) (height - real_maze_height) / 2;
}

void DungeonTypeView::render (Cairo::RefPtr<Cairo::Context> cr, Artwork::Renderer& renderer) {
	cr->save ();
	cr->translate (maze_x, maze_y);
	GridView::render (cr, renderer);

	// still with maze scale from render: get some coordinates
	double en_x = (double) maze->get_entrance ().x;
	double en_y = (double) maze->get_entrance ().y;
	cr->user_to_device_distance (en_x, en_y);
	double dummy = 0, maze_h = (double) maze->height ();
	cr->user_to_device_distance (dummy, maze_h);
	
	// render the reward while still on maze scale
	int icx, icy, icw, ich;
	Glib::RefPtr<DungeonMazeType> dun_maze = Glib::RefPtr<DungeonMazeType>::cast_dynamic (maze);
	dun_maze->cent_room (icx, icy, icw, ich);
	double cx = (double) icx, cy = (double) icy, cw = (double) icw, ch = (double) ich;	
	double x = cw / 2;
	double y = ch / 2;
	
	Cairo::Rectangle rrect = { cx, cy, cw, ch };
	renderer.reward (a_handle, cr, rrect, x, y, Artwork::CENTRE);
	cr->restore ();

	// render the hero
	Cairo::Rectangle hrect = { 0, maze_y, maze_x, maze_h };
	renderer.hero (a_handle, cr, hrect, maze_x, en_y, Artwork::LEFT);
}

void DungeonTypeView::render_solution (Cairo::RefPtr<Cairo::Context> cr, Artwork::Renderer& renderer) {
	cr->save ();
	cr->translate (maze_x, maze_y);
	GridView::render_solution (cr, renderer);

	// still with maze scale from render_soluton
	xy en = maze->get_entrance ();
	cr->move_to (en.x - 0.5, en.y);
	cr->line_to (en.x, en.y);
	cr->stroke ();
	cr->restore ();
}
