/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
/*
 * mazen-ng
 * Copyright (C) Jacob Zimmermann 2009 <jacob@jzimm.net>
 * 
 * mazen-ng is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * mazen-ng is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#pragma implementation

#include "export-dialog.h"
#include "export-png.h"
#include "export-pdf.h"
#include "export-svg.h"
#include "grid-view.h"
#include <stdexcept>
#include <vector>

using namespace Gtk;
using namespace Cairo;
using namespace Glib;
using namespace Gio;
using namespace sigc;

ExportDialog::ExportDialog (BaseObjectType* cobj, const Glib::RefPtr<Gtk::Builder>& builder):
	FileChooserDialog (cobj),
	format_selector (),
	handlers () {

		// init handlers
		std::list<const ExportHandler*> h_list;
		std::list<const ExportHandler*>::const_iterator h_it;
		h_list.push_back (new ExportPDF ());
		h_list.push_back (new ExportSVG ());
		h_list.push_back (new ExportPNG ());

		for (h_it = h_list.begin (); h_it != h_list.end (); ++h_it) {
			ustring desc = (*h_it)->get_description ();
			handlers[desc] = (*h_it);
			format_selector.append_text (desc);
		}
		format_selector.set_active_text (h_list.front ()->get_description ());
		
		set_title ("Save maze");
		set_action (FILE_CHOOSER_ACTION_SAVE);
		set_do_overwrite_confirmation (true);
		set_extra_widget (format_selector);

		adjust_filter ();
		format_selector.signal_changed ().connect (mem_fun (this, &ExportDialog::adjust_filter));
}

void ExportDialog::save (ViewPtr maze_view, int w, int h) {
	if (FileChooserDialog::run () == 1) {
		const ExportHandler* eh = handlers[format_selector.get_active_text ()];
		FOS fos = get_file ()->replace ();
		eh->dump (maze_view, w, h, fos);
		fos->close ();
	}
	hide ();
}

void ExportDialog::adjust_filter () {
	set_filter (handlers[format_selector.get_active_text ()]->get_filter ());
}
