/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
/*
 * mazen-ng
 * Copyright (C) Jacob Zimmermann 2009 <jacob@jzimm.net>
 * 
 * mazen-ng is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * mazen-ng is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#pragma implementation

#include "grid-factory.h"
#include "grid-maze.h"
#include "grid-view.h"

GridMazeFactory::GridMazeFactory (int size, int n_trees, int n_floating_trees) {
	this->size = size;
	this->n_trees = n_trees;
	this->n_floating_trees = n_floating_trees;
}

void GridMazeFactory::set_size (int n) {
	if (size != n) {
		int old_max_trees = get_max_trees ();
		int old_max_floating_trees = get_max_floating_trees ();
		
		size = n;
		size_changed (n);
		
		int new_max_trees = get_max_trees ();
		if (new_max_trees != old_max_trees)
			trees_max_changed(new_max_trees);
		if (n_trees >= new_max_trees) {
			n_trees = new_max_trees - 1;
			trees_changed (n_trees);
		}

		int new_max_floating_trees = get_max_floating_trees ();
		if (new_max_floating_trees != old_max_floating_trees)
			floating_trees_max_changed (new_max_floating_trees);
		if (n_floating_trees >= new_max_floating_trees) {
			n_floating_trees = new_max_floating_trees - 1;
			floating_trees_changed (n_floating_trees);
		}
	}
}

void GridMazeFactory::set_trees (int n) {
	if (n_trees != n) {
		int old_max_floating_trees = get_max_floating_trees ();		
		n_trees = n;
		trees_changed (n);

		int new_max_floating_trees = get_max_floating_trees ();
		if (new_max_floating_trees != old_max_floating_trees)
			floating_trees_max_changed (new_max_floating_trees);
		if (n_floating_trees > new_max_floating_trees) {
			n_floating_trees = new_max_floating_trees;
			floating_trees_changed (n_floating_trees);
		}
	}
}

void GridMazeFactory::set_floating_trees (int n) {
	if (n_floating_trees != n) {
		n_floating_trees = n;
		floating_trees_changed (n_floating_trees);		
	}
}

int GridMazeFactory::get_max_floating_trees () const {
	return get_max_trees () - n_trees;
}