/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
/*
 * mazen-ng
 * Copyright (C) Jacob Zimmermann 2009 <jacob@jzimm.net>
 * 
 * mazen-ng is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * mazen-ng is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef _GRID_FACTORY_H_
#define _GRID_FACTORY_H_

#pragma interface

#include <glibmm.h>

class GridMaze;
class GridView;

class GridMazeFactory
{
	protected:
		int size, n_trees, n_floating_trees;
		GridMazeFactory (int size, int n_trees, int n_floating_trees);
		
	public:
		sigc::signal<void,int> size_changed;
		sigc::signal<void,int> trees_max_changed, trees_changed;
		sigc::signal<void,int> floating_trees_max_changed, floating_trees_changed;		
		
		virtual Glib::ustring get_name() const = 0;
		virtual Glib::RefPtr<GridView> new_maze() const = 0;

		virtual void set_size (int n);
		int get_size () const { return size; }
		virtual void set_trees (int n);
		int get_trees () const { return n_trees; }
		virtual void set_floating_trees (int n);
		int get_floating_trees () const { return n_floating_trees; }
		
		virtual int get_min_trees() const = 0;
		virtual int get_max_trees() const = 0;
		virtual int get_max_floating_trees() const;
};


#endif // _GRID_FACTORY_H