/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
/*
 * mazen-ng
 * Copyright (C) Jacob Zimmermann 2009 <jacob@jzimm.net>
 * 
 * mazen-ng is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * mazen-ng is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#pragma implementation

#include "grid-maze.h"
#include "grid-view.h"
#include "util.h"

using namespace Glib;

void GridView::calculate_zfact () {
	double mx_maze = MAX((double) maze->width () - 1, (double) maze->height () - 1);
	double mn_dev = MIN((double) width, (double) height);

	zfact = mn_dev / mx_maze;
	changed = true;
}

GridView::GridView () {
	a_handle = Artwork::next ();
	changed = false;
}

void GridView::set_maze (MazePtr m) {
	maze = m;
	calculate_zfact ();
}

void GridView::set_size (int width, int height) {
	this->width = width;
	this->height = height;
	calculate_zfact ();
}

ViewPtr GridView::create () {
	return ViewPtr (new GridView ());
}

void GridView::render (Cairo::RefPtr<Cairo::Context> cr, Artwork::Renderer& renderer) {
	cr->set_source_rgb (0, 0, 0);
	cr->scale (zfact, zfact);
	set_line_width (cr, 1);

	// render grid
	for (int x = 0; x < maze->width (); x++) {
		for (int y = 0; y < maze->height (); y++) {
			if (maze->vert (x, y)) {
				cr->move_to (x, y + 1);
				cr->line_to (x, y);
			}
			else
				cr->move_to (x, y);
			if (maze->horiz (x, y))
				cr->line_to (x + 1, y);
		}
		cr->stroke ();
	}

	// render the walls
	const std::list<GridMaze::Wall>& walls = maze->get_walls ();
	std::list<GridMaze::Wall>::const_iterator wall;
	GridMaze::Wall::const_iterator pt;

	set_line_width (cr, 2);
	for (wall = walls.begin (); wall != walls.end (); ++wall) {
		pt = wall->begin ();
		cr->move_to (pt->x, pt->y);
		while (++pt != wall->end ())
			cr->line_to (pt->x, pt->y);
		cr->stroke ();
	}
	
	changed = false;
}

void GridView::render_solution (Cairo::RefPtr<Cairo::Context> cr, Artwork::Renderer& renderer) {
	const std::list<xy>& solution = maze->get_solution ();

	cr->set_source_rgb (1, 0, 0);
	cr->scale (zfact, zfact);
	cr->translate (0.5, 0.5);
	set_line_width (cr, 1);

	xy ex = maze->get_exit ();
	std::list<xy>::const_iterator p = solution.begin ();
	std::list<xy>::const_iterator last = solution.end ();
	last --;
	
	cr->move_to (p->x, p->y);
	while ((++p) != solution.end ())
		cr->line_to (p->x, p->y);
	switch (maze->get_exit_dir ()) {
		case GridMaze::NONE:
			cr->line_to (ex.x, ex.y);
			break;
		case GridMaze::LEFT:
			cr->line_to (ex.x, ex.y);
			cr->line_to (ex.x - 0.5, ex.y);			
			break;
		case GridMaze::RIGHT:
			cr->line_to (last->x + 0.5, last->y);
			break;
		case GridMaze::UP:
			cr->line_to (last->x, last->y + 0.5);
			break;
		case GridMaze::DOWN:
			cr->line_to (ex.x, ex.y);
			cr->line_to (ex.x, ex.y - 0.5);
			break;
	}
	cr->stroke ();
}

