/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
/*
 * mazen-ng
 * Copyright (C) Jacob Zimmermann 2009 <jacob@jzimm.net>
 * 
 * mazen-ng is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * mazen-ng is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#pragma implementation

#include "polygon-maze.h"
#include "polygon-walker.h"
#include <algorithm>

using namespace std;

PolygonMaze::PolygonMaze (int size, int faces, int n_trees,  int n_floating_trees): GridMaze (size, faces) {
	vector<xy> trees;

	// Border trees
	for (int y = 0; y < height (); y++) {
		set_vert (0, y);
		set_vert (size - 1, y);
		trees.push_back (xy (0, y));
		trees.push_back (xy (size - 1, y));
	}

	int fx, fy;
	for (int k = 0; k < n_floating_trees; k++) {
		fx = g_random_int_range (0, size);
		fy = g_random_int_range (1, faces - 1);
		trees.push_back (xy (fx, fy));
	}

	clear_vert (get_entrance ());
	clear_vert (get_exit ());
	
	random_shuffle (trees.begin (), trees.end ());

	for (int i = 0; i < n_trees; i++)
		tree_positions.push_back (trees[i]);

	generate ();
}

void PolygonMaze::possible_directions (xy pos, vector<Direction>& dirs) const {
	// test up
	int ny = (pos.y + 1) % height ();
	if (!vert (pos)
	    && !is_no_no (pos)
	    && is_clear (pos.x, ny)
	    && ((pos.x == 0) || !horiz (pos.x - 1, ny)))
		
		dirs.push_back (UP);

	// test down
	if (pos.y > 1) {
	    if (is_clear (pos.down ())
	        && (!vert (pos.x, pos.y - 2))
	        && (pos.x == 0 || !horiz (pos.x - 1, pos.y - 1)))

			dirs.push_back (DOWN);
	}
	else if (pos.y == 1) {
		// wrap down by 1
		if (is_clear (pos.down ())
		    && (!vert (pos.x, height () - 1))
		    && (pos.x == 0 || !horiz (pos.x - 1, 0)))

			dirs.push_back (DOWN);
	}
	else {
		// wrap down by 2
		int ny = height () - 1;
	    if (is_clear (pos.x, ny)
	        && (!vert (pos.x, ny - 1))
	        && (pos.x == 0 || !horiz (pos.x - 1, ny)))

			dirs.push_back (DOWN);
	}

	// test left
	if (pos.x > 0
	    && is_clear (pos.left ())
	    && (pos.x == 1 || !horiz (pos.x - 2, pos.y))
	    && ((pos.y > 0 && !vert (pos.x - 1, pos.y - 1))
	        || (pos.y == 0 && !vert (pos.x - 1, height () - 1))))

		dirs.push_back (LEFT);

	// test right
	if (pos.x < width () - 1
	    && !horiz (pos)
	    && !is_no_no (pos)
	    && is_clear (pos.right ())
	    && ((pos.y > 0 && !vert (pos.x + 1, pos.y - 1))
	        || (pos.y == 0 && !vert (pos.x + 1, height () - 1))))

		dirs.push_back (RIGHT);
}

void PolygonMaze::move (xy& pos, Direction dir) {
	switch(dir) {
		case UP:
			set_vert (pos.x, pos.y);
			pos.y ++;
			pos.y %= height ();
			break;
		case DOWN:
			if (pos.y == 0)
				pos.y = height () - 1;
			set_vert (pos.x, --pos.y);
			break;
		case RIGHT:
			set_horiz (pos.x++, pos.y);
			break;
		case LEFT:
			set_horiz (--pos.x, pos.y);
			break;
	}
}

xy PolygonMaze::get_entrance () const {
	return xy (width () - 1, height () / 2);
}

xy PolygonMaze::get_exit () const {
	return xy (0, 0);
}

void PolygonMaze::solve () {
	PolygonWalker walker (*this);
	run_walker (walker);
}

