/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
/*
 * mazen-ng
 * Copyright (C) Jacob Zimmermann 2009 <jacob@jzimm.net>
 * 
 * mazen-ng is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * mazen-ng is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#pragma implementation

#include "grid-maze.h"
#include "polygon-view.h"
#include <list>

PolygonView::PolygonView (): PolarView () {
}

void PolygonView::cline (Cairo::RefPtr<Cairo::Context> cr, int y, double r, PolarView::cline_type ltype) const {
	double g = gap ();

	y %= maze->height ();
	int ym1 = (y - 1);
	if (ym1 < 0)
		ym1 += maze->height ();
	
	double dx1 = r, dy1 = g;
	point_coord (ym1, dx1, dy1);
	
	double dx2 = r, dy2 = -g;
	point_coord (y, dx2, dy2);
	
	double dx3 = r, dy3 = g;
	double dx4, dy4;
	double dx5, dy5;

	switch (ltype) {
		case OPEN:
			cr->move_to (dx1, dy1);
			cr->line_to (dx2, dy2);
			break;
			
		case CLOSED:
			cr->move_to (dx1, dy1);
			cr->line_to (dx2, dy2);
			
			point_coord (y, dx3, dy3);
			cr->line_to (dx3, dy3);
			break;

		default:
			dx4 = dx5 = r;
			dy4 = dy5 = 0;
			point_coord (ym1, dx4, dy4);
			point_coord (y, dx5, dy5);
			
			cr->move_to (dx4, dy4);
			cr->line_to (dx1, dy1);
			cr->line_to (dx2, dy2);
			cr->line_to (dx5, dy5);
			break;
	}
}

ViewPtr PolygonView::create () {
	return ViewPtr (new PolygonView ());
}