/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
/*
 * mazen-ng
 * Copyright (C) Jacob Zimmermann 2009 <jacob@jzimm.net>
 * 
 * mazen-ng is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * mazen-ng is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#pragma implementation

#include "traversal-maze.h"
#include <algorithm>

TraversalMaze::TraversalMaze (int width, int height, int n_trees, int n_floating_trees):
	GridMaze (width, height),
	entrance (0, g_random_int_range (1, height - 2)),
	exit (width - 1, g_random_int_range (1, height - 2)) {

		exit_dir = GridMaze::RIGHT;
		
		// create border wall
		for (int x = 0; x < width - 1; x++) {
			set_horiz (x, 0);
			set_horiz (x, height - 1);
		}
		for (int y = 0; y < height - 1; y++) {
			set_vert (0, y);
			set_vert (width - 1, y);
		}

		Wall upper, lower;

		upper.push_back (entrance);
		upper.push_back (xy (0, 0));
		upper.push_back (xy (width - 1, 0));
		upper.push_back (exit);

		lower.push_back (exit.up ());
		lower.push_back (xy (width - 1, height - 1));
		lower.push_back (xy (0, height - 1));
		lower.push_back (entrance.up ());
		
		walls.push_back (upper);
		walls.push_back (lower);
		
		// entrance and exit
		clear_vert (entrance);
		clear_vert (exit);

		// border trees coordinates
		std::vector<xy> t (2 * (width + height) - 8);
		int p = 0;
		for (int x = 0; x < width - 2; x++) {
			t[p++].set (x + 1, 0);
			t[p++].set (x + 1, height - 1);
		}
		for (int y = 0; y < height - 2; y++) {
			t[p++].set (0, y + 1);
			t[p++].set (width - 1, y + 1);
		}
		random_shuffle (t.begin (), t.end ());
		// create border trees
		for (int k = 0; k < n_trees; k++)
			tree_positions.push_back (t[k]);

		// create floating trees
		int x, y;
		for (int k = 0; k < n_floating_trees; k++) {
			x = g_random_int_range (1, width - 2);
			y = g_random_int_range (1, height - 2);
			if (is_clear (x, y))
				tree_positions.push_back (xy (x, y));
		}

		generate();
	}

xy TraversalMaze::get_entrance () const { return entrance; }
xy TraversalMaze::get_exit () const { return exit; }
